@@ -9,8 +9,8 @@ import hmac
 import time
 
 from jinja2.exceptions import TemplateNotFound
-from shake import (url_for, from36, to36, redirect, Rule, StorageDict,
-    to_bytestring)
+from shake import (url_for, from36, to36, redirect_to, Rule, StorageDict,
+    to_bytestring,)
 
 try:
     import mailshake
@@ -18,8 +18,9 @@ except ImportError:
     pass
 
 from .perms import REDIRECT_FIELD_NAME, is_valid
-from .utils import *
-from .utils import bcrypt, scrypt, hash_old_sha256, hash_old_sha512
+from .utils import (bcrypt, scrypt, split_hash, generate_salt, pbkdf2,
+    hash_sha256, hash_sha512, hash_bcrypt, hash_scrypt,
+    get_user_model, LazyUser, Token, auth_render)
 
 
 __all__ = (
@@ -156,11 +157,11 @@ class Auth(object):
         'password_minlen': 5,
         'reset_expire': 3,  # hours
 
-        'sign_in_template': 'auth_sign_in.html',
+        'sign_in_template': 'auth/sign_in.html',
         'sign_out_template': None,
-        'reset_password_template': 'auth_reset_password.html',
-        'email_reset_template': 'auth_email_reset_password.html',
-        'change_password_template': 'auth_change_password.html',
+        'reset_password_template': 'auth/reset_password.html',
+        'email_reset_template': 'auth/email_reset_password.html',
+        'change_password_template': 'auth/change_password.html',
 
         'render': None,
         'mailer': None,
@@ -221,7 +222,9 @@ class Auth(object):
         
         assert settings.reset_expire >= 0, \
             "`reset_expire` parameter must be a positive number"
-    
+        
+        self.render = settings.pop('render', None)
+        self.mailer = settings.pop('mailer', None)
         self.settings = settings
         
         if app:
@@ -233,6 +236,7 @@ class Auth(object):
         use with this authentication setup.
         """
         self.app = app
+        self.render = self.render or app.render
         
         @app.before_request
         def set_user(request):
@@ -317,8 +321,8 @@ class Auth(object):
 
     def hash_password(self, password):
         password += self.settings.pepper
-        password = to_bytestring(password)
-        return self.hash_func(password, cost=self.settings.hash_cost)
+        hashed = self.hash_func(password, cost=self.settings.hash_cost)
+        return hashed
     
 
     def check_password(self, password, stored_password):
@@ -329,27 +333,26 @@ class Auth(object):
         if stored_password is None:
             return False
 
-        password = to_bytestring(password)
         stored_password = to_bytestring(stored_password)
+        password += self.settings.pepper
         try:
             hash_alg, hash_cost, salt = split_hash(stored_password)
         except ValueError:
             raise ValueError("The stored password isn't a valid hash")
-        
-        password += self.settings.pepper
 
         if hash_alg == 'scrypt':
             assert 'scrypt' in globals().keys(), ('\'scrypt\' is not '
                 'available but the stored password was hashed using it.\n '
                 'Please install scrypt '
                 '<http://pypi.python.org/pypi/scrypt/>')
-            hashed = hash_scrypt(password, stored_password, hash_cost)
+            hashed = hash_scrypt(password, salt, hash_cost)
 
-        if hash_alg == 'bcrypt':
+        elif hash_alg == 'bcrypt':
             assert 'bcrypt' in globals().keys(), ('\'bcrypt\' is not '
                 'available but the stored password was hashed using it.\n '
                 'Please install py-bcrypt '
                 '<http://pypi.python.org/pypi/py-bcrypt/>')
+            hashed = hash_scrypt(password, stored_password, hash_cost)
 
         elif hash_alg == 'dsha512':
             hashed = hash_sha512(password, salt, hash_cost)
@@ -365,11 +368,11 @@ class Auth(object):
         
         else:
             raise ValueError("Hash algorithm '%s' not available" % hash_alg)
+
         return stored_password == hashed
     
 
     def set_password(self, user, passw):
-        passw = passw.encode('utf8', 'ignore')
         if len(passw) < self.settings.password_minlen:
             raise PasswordTooShortError(self.settings.password_minlen)
         user.password = self.hash_password(passw)
@@ -415,17 +418,14 @@ class Auth(object):
         Creates a new user.
         This method is intended to be called from the command line.
         """
-        login = login.encode('utf8', 'ignore')
-        passw = passw.encode('utf8', 'ignore')
+        login = to_bytestring(login)
+        passw = to_bytestring(passw)
         user_exists = self._get_user(login)
         if user_exists:
             raise UserExistsError(login)
-        if len(passw) < self.settings.password_minlen:
-            raise PasswordTooShortError(self.settings.password_minlen)
-        
-        passw = self.hash_password(passw)
-        user = self.User(login=login, password=passw, **data)
+        user = self.User(login=login, password='', **data)
         self.db.add(user)
+        self.set_password(user, passw)
         self.db.commit()
         return user
     
@@ -435,8 +435,8 @@ class Auth(object):
         Changes the password of an existing user.
         This method is intended to be called from the command line.
         """
-        login = login.encode('utf8', 'ignore')
-        passw = passw.encode('utf8', 'ignore')
+        login = to_bytestring(login)
+        passw = to_bytestring(passw)
         user = self._get_user(login)
         if not user:
             raise UserDoesNotExistsError(login)
@@ -445,16 +445,16 @@ class Auth(object):
         return user
     
 
-    def render(self, template, context):
+    def render_template(self, template, context):
         try:
-            return self.settings.render(template, context)
+            return self.render(template, context)
         except TemplateNotFound:
             return auth_render(template, context)
     
 
     def render_to_string(self, template, context):
         try:
-            return self.settings.render(template, context, to_string=True)
+            return self.render(template, context, to_string=True)
         except TemplateNotFound:
             return auth_render(template, context, to_string=True)
     
@@ -462,15 +462,15 @@ class Auth(object):
     def sign_in_view(self, request, **kwargs):
         """Signs a user in.
         """
-        redirect_to = request.session.get(REDIRECT_FIELD_NAME)
-        redirect_to = redirect_to or self.settings.sign_in_redirect
-        if callable(redirect_to):
-            redirect_to = redirect_to()
+        redirect_url = request.session.get(REDIRECT_FIELD_NAME)
+        redirect_url = redirect_url or self.settings.sign_in_redirect
+        if callable(redirect_url):
+            redirect_url = redirect_url()
         
         # Redirect if there's already a signed user
         if request.user:
             request.session.pop(REDIRECT_FIELD_NAME, None)
-            return redirect(redirect_to)
+            return redirect_to(redirect_url)
         
         credentials = request.form
         kwargs['auth'] = self
@@ -484,32 +484,32 @@ class Auth(object):
                     user.last_sign_in = datetime.utcnow()
                     self.db.commit()
                 self.sign_in(request, user)
-                return redirect(redirect_to)
+                return redirect_to(redirect_url)
             kwargs['error'] = 'FAIL'
         
         template = self.settings.sign_in_template
-        return self.render(template, kwargs)
+        return self.render_template(template, kwargs)
     
 
     def sign_out_view(self, request, **kwargs):
         """Signs out a user.
         
         """
-        redirect_to = request.session.get(REDIRECT_FIELD_NAME)
-        redirect_to = redirect_to or self.settings.sign_out_redirect
-        if callable(redirect_to):
-            redirect_to = redirect_to()
+        redirect_url = request.session.get(REDIRECT_FIELD_NAME)
+        redirect_url = redirect_url or self.settings.sign_out_redirect
+        if callable(redirect_url):
+            redirect_url = redirect_url()
         
         self.sign_out(request)
         
         template = self.settings.sign_out_template
         if template:
             kwargs['auth'] = self
-            return self.render(template, kwargs)
-        return redirect(redirect_to)
+            return self.render_template(template, kwargs)
+        return redirect_to(redirect_url)
     
 
-    def change_password_view(self, request, token=None, **kwargs):
+    def reset_password_view(self, request, token=None, **kwargs):
         """A apge to generate a link to reset your password.
         
         token
@@ -522,7 +522,7 @@ class Auth(object):
         kwargs['error'] = None
         
         if not token and request.user:
-            return redirect(url_for('auth.change_password'))
+            return redirect_to(url_for('auth.change_password'))
         
         # Phase 3
         if token:
@@ -532,7 +532,7 @@ class Auth(object):
                 if request.user and (request.user != user):
                     self.sign_out(request)
                 self.sign_in(request, user)
-                return self.reset_password_view(request, user_requested=False)
+                return self.change_password_view(request, user_requested=False)
             
             kwargs['error'] = 'WRONG TOKEN'
         
@@ -544,24 +544,30 @@ class Auth(object):
                 kwargs['error'] = 'NOT FOUND'
             else:
                 # User founded. Send token by email
+                site_name = self.app.settings.get('SITE_NAME')
+                site_name = site_name or getattr(request, 'host')
                 token = self.get_reset_token(user)
-                self.email_token(user, token)
+                self.email_token(user, token, site_name)
                 kwargs['ok'] = True
         
         template = self.settings.reset_password_template
-        return self.render(template, kwargs)
+        return self.render_template(template, kwargs)
     
 
-    def email_token(self, user, token):
+    def email_token(self, user, token, site_name):
+        assert self.mailer
         template = self.settings.email_reset_template
-        self.render_to_string(template, {'user': user, 'token': token})
-        msg = msg.encode('utf-8')
-        
-        mailer = self.settings.mailer
+        msg = self.render_to_string(template, {
+            'user': user,
+            'token': token,
+            'site_name': site_name,
+        })
+        msg = to_bytestring(msg)
+
         email_msg = mailshake.EmailMessage(
             subject='Reset your password',
             text_content='',
-            from_email=mailer.default_from,
+            from_email=self.mailer.default_from,
             to=user.email
         )
         if template.endswith('.html'):
@@ -569,14 +575,14 @@ class Auth(object):
         else:
             email_msg.text_content = msg
 
-        mailer.send(email_msg)
+        self.mailer.send(email_msg)
     
 
-    def reset_password_view(self, request, user_requested=True, **kwargs):
+    def change_password_view(self, request, user_requested=True, **kwargs):
         """A view to change your password.
         """
         if not is_valid(request):
-            return redirect(url_for('auth.sign_in'))
+            return redirect_to(url_for('auth.sign_in'))
         
         sign_in_redirect = self.settings.sign_in_redirect
         if callable(sign_in_redirect):
@@ -590,9 +596,9 @@ class Auth(object):
         
         if request.is_post:
             data = request.form
-            password = data.get('password', '')
-            np1 = data.get('new_password_1', '')
-            np2 = data.get('new_password_2', '')
+            password = to_bytestring(data.get('password', ''))
+            np1 = to_bytestring(data.get('new_password_1', ''))
+            np2 = to_bytestring(data.get('new_password_2', ''))
             
             # Validate the new password
             if len(np1) < self.settings.password_minlen:
@@ -615,7 +621,7 @@ class Auth(object):
                 kwargs['ok'] = True
         
         template = self.settings.change_password_template
-        return self.render(template, kwargs)
+        return self.render_template(template, kwargs)
 
 
     sign_in_controller = sign_in_view
