@@ -11,7 +11,7 @@ from solution import SQLAlchemy
 
 
 app_settings = {
-    'secret_key': 'ee6864874322ff97d0f7380e815ff0a02493783c',
+    'SECRET_KEY': 'ee6864874322ff97d0f7380e815ff0a02493783c',
 }
 
 auth_settings = {
@@ -31,6 +31,7 @@ def get_auth(hash_alg='sha512', hash_cost=10):
     settings = auth_settings.copy()
     settings['hash_alg'] = hash_alg
     settings['hash_cost'] = hash_cost
+    settings['pepper'] = 'dc47d3062962d5b9f5828dccc47b9d7eda95ecf6'
     return shake_auth.Auth(User, **settings)
 
 
@@ -68,7 +69,7 @@ def test_sign_in():
     def get_user(request):
         return request.user.login
 
-    app = Shake(app_settings)
+    app = Shake(__file__, app_settings)
     auth.init_app(app)
     app.add_url('/', index)
     app.add_url('/user/', get_user)
@@ -94,7 +95,7 @@ def test_sign_out():
         auth.sign_out(request)
         assert request.user is None
 
-    app = Shake(app_settings)
+    app = Shake(__file__, app_settings)
     auth.init_app(app)
     app.add_url('/', index)
     app.add_url('/sign_out/', sign_out)
@@ -129,7 +130,7 @@ def test_authenticate_by_password():
 
 def test_authenticate_by_token():
     auth = get_auth()
-    app = Shake(app_settings)
+    app = Shake(__file__, app_settings)
     auth.init_app(app)
 
     user = auth.create_user(u'admin', u'passw')
@@ -149,6 +150,7 @@ def test_authenticate_and_update():
     settings = auth_settings.copy()
     settings['hash_alg'] = 'sha512'
     settings['hash_cost'] = 10
+    settings['pepper'] = 'dc47d3062962d5b9f5828dccc47b9d7eda95ecf6'
     auth = shake_auth.Auth(User, **settings)
 
     settings = settings.copy()
